# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['mobile_strings_converter']

package_data = \
{'': ['*'], 'mobile_strings_converter': ['assets/fonts/*']}

install_requires = \
['arabic-reshaper>=3.0.0,<4.0.0',
 'fpdf2>=2.6.1,<3.0.0',
 'gspread>=5.7.2,<6.0.0',
 'lingua-language-detector>=1.3.2,<2.0.0',
 'openpyxl>=3.1.0,<4.0.0',
 'protobuf>=4.21.12,<5.0.0',
 'python-bidi>=0.4.2,<0.5.0',
 'pyyaml>=6.0,<7.0']

setup_kwargs = {
    'name': 'mobile-strings-converter',
    'version': '0.1.0',
    'description': 'A Python package that converts Android & iOS strings files to any supported file type.',
    'long_description': '<div id="top"></div>\n\n<!-- PROJECT SHIELDS -->\n<!--\n*** I\'m using markdown "reference style" links for readability.\n*** Reference links are enclosed in brackets [ ] instead of parentheses ( ).\n*** See the bottom of this document for the declaration of the reference variables\n*** for contributors-url, forks-url, etc. This is an optional, concise syntax you may use.\n*** https://www.markdownguide.org/basic-syntax/#reference-style-links\n-->\n\n<!-- PROJECT LOGO -->\n<br />\n<div align="center">\n    <img src="docs/icon.png" alt="Logo" width="156" height="156" style="margin-bottom:-40px">\n    <h2 align="center">Mobile strings converter</h2>\n    <p align="center">\n        A Python package that converts Android & iOS strings files to any supported file type.\n        <br />\n        <br />\n        <a href="https://pypi.org/project/mobile-strings-converter/">\n          <img alt="PyPI version" src="https://img.shields.io/pypi/v/mobile-strings-converter" />\n        </a>\n        <a href="https://pypi.org/project/mobile-strings-converter/">\n          <img alt="Python versions support" src="https://img.shields.io/pypi/pyversions/mobile-strings-converter" />\n        </a>\n        <br />\n        <a href="https://github.com/HenestrosaConH/mobile-strings-converter/actions/workflows/build.yaml">\n          <img alt="GitHub action: Build" src="https://github.com/HenestrosaConH/mobile-strings-converter/actions/workflows/build.yaml/badge.svg" />\n        </a>\n        <a href="https://codecov.io/gh/HenestrosaConH/android-strings-converter/">\n          <img alt="Codecov" src="https://codecov.io/gh/HenestrosaConH/android-strings-converter/branch/main/graph/badge.svg" />\n        </a>\n        <br />\n        <a href="https://github.com/HenestrosaConH/mobile-strings-converter/graphs/contributors">\n          <img alt="GitHub Contributors" src="https://img.shields.io/github/contributors/HenestrosaConH/mobile-strings-converter" />\n        </a>\n        <a href="https://github.com/HenestrosaConH/mobile-strings-converter/blob/main/LICENSE">\n          <img alt="License" src="https://img.shields.io/github/license/HenestrosaConH/mobile-strings-converter" />\n        </a>\n        <a href="https://github.com/HenestrosaConH/mobile-strings-converter/issues">\n          <img alt="Issues" src="https://img.shields.io/github/issues/HenestrosaConH/mobile-strings-converter" />\n        </a>\n        <a href="https://github.com/HenestrosaConH/mobile-strings-converter/pulls">\n          <img alt="GitHub pull requests" src="https://img.shields.io/github/issues-pr/HenestrosaConH/mobile-strings-converter" />\n        </a>\n        <br />\n        <br />\n        <a href="https://github.com/HenestrosaConH/mobile-strings-converter/issues/new/choose">Report Bug</a> · <a href="https://github.com/HenestrosaConH/mobile-strings-converter/issues/new/choose">Request Feature</a> · <a href="https://github.com/HenestrosaConH/mobile-strings-converter/discussions">Ask Question</a>\n    </p>\n</div>\n\n<!-- TABLE OF CONTENTS -->\n\n## Table of Contents\n\n- [About The Project](#about-the-project)\n    - [Project Structure](#project-structure)\n    - [Built With](#built-with)\n- [Usage](#usage)\n    - [To execute the script](#to-execute-the-script)\n    - [To import the package into your project](#to-import-the-package-into-your-project)\n    - [To open the code](#to-open-the-code)\n- [Roadmap](#roadmap)\n- [Contributing](#contributing)\n- [License](#license)\n- [Authors](#authors)\n- [Acknowledgments](#acknowledgments)\n\n<!-- ABOUT THE PROJECT -->\n\n## About The Project\n\nI tried to do the whole process of converting a **strings.xml** file into a spreadsheet in Google Sheets by hand and, even though you can do it with the option **Data > Split text to columns**, \nit involves wasting your time generating the spreadsheet manually instead of the more time-efficient solution of running a Python script to achieve that with any file type.\n\nMoreover, not only this script can be executed on its own, it also can be installed as a package via **PyPI** (more information [here](#to-import-the-package-into-your-project) about how to install it).\n\nThe file types supported by the package are the following:\n- Android strings format (strings.xml)\n- CSV\n- Google Sheets support\n- HTML\n- iOS strings format (Localizable.strings)\n- JSON\n- MD\n- ODS\n- PDF\n- XLSX\n- YAML\n\n<!-- PROJECT STRUCTURE -->\n\n### Project Structure\n\nRoot directories:\n\n- `.github/workflows`: GitHub workflows. It also includes the templates for issues and pull requests, as well as the `depandabot.yml` file for Dependabot configuration.\n- `docs`: Contains files related to the documentation of the project.\n- `src/mobile_strings_converter`:  Contains the source code files.\n- `tests`: Contains unit tests to ensure the correct functionality of the package. It also includes the `files` directory, which contains a few demo files in different formats to use in the unit tests.\n\nRoot files:\n\n- `.gitignore`: File used by the version control system Git to specify files or directories that should be ignored by Git when tracking changes to a project.\n- `.pre-commit-config.yaml`: Configuration file used by **pre-commit**, a tool that runs checks (such as linting, testing, or formatting) on the code before you commit changes to version control. The file specifies which checks pre-commit should run and how it should run them.\n- `LICENSE`: Project license, which is [MIT](https://opensource.org/license/mit/).\n- `poetry.lock`: File generated by **Poetry**, a package manager for Python, that contains the exact versions of all packages used by a project. The file is used to ensure that all members of a development team are using the same versions of packages, even if different versions are available in the package repository.\n- `pyproject.toml`: Configuration file used by **Poetry**. It specifies the metadata for a Python project, including the project name, version, description, author, license and dependencies.\n- `README.md`: What you\'re reading right now.\n- `requirements.txt`: Lists the names and versions of each package used to build this project. To install the requirements, execute `pip install -r requirements.txt`.\n- `requirements-dev.txt`: Lists the names and versions of each package used in the development stage of this project. To install the requirements, execute `pip install -r requirements-dev.txt`.\n\n\n<!-- BUILT WITH -->\n\n### Built With\n\n- [openpyxl](https://pypi.org/project/openpyxl/): To generate ODS and XLSX files.\n- [gspread](https://pypi.org/project/gspread/): To generate spreadsheets in Google Sheets.\n- [protobuf](https://pypi.org/project/oauth2client/): Used by `google.oauth2.credentials` to authenticate to the user\'s Google account in order to create the spreadsheet in Google Sheets. \n- [PyYAML](https://pypi.org/project/PyYAML/): To generate YAML files.\n- [arabic-reshaper](https://pypi.org/project/arabic-reshaper/) and [python-bidi](https://pypi.org/project/python-bidi/): To add arabic characters support for PDF files.\n- [fpdf2](https://pypi.org/project/fpdf2/): To generate PDF files.\n- [lingua-language-detector](https://pypi.org/project/lingua-language-detector/): To recognize the **value** language when writing a PDF in order to know what font to use.  \n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n\n<!-- USAGE -->\n\n## Usage\n\n### To execute the script\n1. Go to [releases](https://github.com/HenestrosaConH/mobile-strings-converter/releases) \n2. Download the latest one.\n3. Open the command line and execute `pip install -r path/to/requirements.txt` to install the required packages to execute the program.\n4. Run the following command: \n    ```\n    path/to/python path/to/mobile_strings_converter.py path/to/<strings.xml | Localizable.strings> -o path/to/strings.<SUPPORTED FILE TYPE EXTENSION>\n    ```\n    This will generate a file named `strings` in the desired path.  \n    <br>\n\nIf you want to generate a spreadsheet in Google Sheets, run the following command:\n```\npath/to/python path/to/mobile_strings_converter.py path/to/<strings.xml | Localizable.strings> -gs <SHEET NAME> -c path/to/service_account.json \n```\n\nIf you want to generate an output file along with a spreadsheet in Google Sheets, run the following command:\n```\npath/to/python path/to/mobile_strings_converter.py path/to/<strings.xml | Localizable.strings> -gs <SHEET NAME> -c path/to/service_account.json -o path/to/strings.<SUPPORTED FILE TYPE EXTENSION>\n```\n\nPlease note that you will have to generate the `service_account.json` yourself. To do it, you can do the following:\n\n1. Go to the Google Cloud Console: https://console.cloud.google.com/\n2. Create a new project or select an existing project.\n3. Go to the **APIs & Services** page, click on **Dashboard** and then click on **Enable APIs and Services**.\n4. Search for **Google Sheets API** and enable it.\n5. Go to the **Credentials** page, click on **Create credentials**, and then choose **Service account**.\n6. Give your service account a name and select a role. For this purpose, you can select **Project -> Editor**.\n7. Click on the "Create key" button, select the JSON format and download the `service_account.json` file.\n8. Share your Google Sheets file with the email address that is specified in the **client_email** field in the `service_account.json` file.\n\nAlternatively, you can create an XLSX file and open it in Google Sheets if you don\'t want to go through the hassle of generating the `service_account.json` file.\n\n### To import the package into your project\n1. Run `pip install mobile-strings-converter`\n2. Import the package with this line of code: `from mobile_strings_converter import <FUNCTION>`\n   - For example, if you were to import the `to_csv` function: `from mobile_strings_converter import to_csv`\n\n### To open the code\n1. Clone the project with the `git clone https://github.com/HenestrosaConH/mobile-strings-converter.git` command.\n2. Open it in your favourite IDE (mine is [PyCharm](https://www.jetbrains.com/pycharm/))\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n## Roadmap\n\n- [ ] Make a web version with Django/Flask.\n\nYou can propose a new feature creating an [issue](https://github.com/HenestrosaConH/mobile-strings-converter/new/choose).\n\n<p align="right">(<a href="#readme-top">back to top</a>)</p>\n\n<!-- CONTRIBUTING -->\n\n## Contributing\n\nContributions are what make the open source community such an amazing place to learn, inspire, and create. Any contributions you make are **greatly appreciated**.\nPlease, read the [CONTRIBUTING.md](https://github.com/HenestrosaConH/mobile-strings-converter/blob/main/.github/CONTRIBUTING.md) file, where you can find more detailed information about how to contribute to the project.\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n<!-- LICENSE -->\n\n## License\n\nDistributed under the MIT License. See `LICENSE` for more information.\n\n<p align="right">(<a href="#readme-top">back to top</a>)</p>\n\n<!-- AUTHORS -->\n\n## Authors\n\n- HenestrosaConH <henestrosaconh@gmail.com> (José Carlos López)\n\nSee also the list of [contributors](https://github.com/HenestrosaConH/mobile-strings-converter/contributors) who participated in this project.\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n<!-- ACKNOWLEDGMENTS -->\n\n## Acknowledgments\n\nI\'ve made use of the following resources to make this project:\n\n- [Best-README-Template](https://github.com/othneildrew/Best-README-Template/)\n- [Img Shields](https://shields.io)\n- [How to create a Python package](https://mathspp.com/blog/how-to-create-a-python-package-in-2022#how-to-create-a-python-package)\n- [Icon created by Midjourney](https://www.midjourney.com/app/)\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n',
    'author': 'José Carlos López',
    'author_email': 'henestrosaconh@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/HenestrosaConH/mobile-strings-converter',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8',
}


setup(**setup_kwargs)
